"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const rimraf = require("rimraf-then");
const path = require("path");
const seq = require("promisequence");
const RegClient = require("npm-registry-client");
const pnpm_logger_1 = require("pnpm-logger");
const cloneDeep = require("lodash.clonedeep");
const got_1 = require("../network/got");
const getContext_1 = require("./getContext");
const installMultiple_1 = require("../install/installMultiple");
const save_1 = require("../save");
const linkPeers_1 = require("../install/linkPeers");
const runtimeError_1 = require("../runtimeError");
const getSaveType_1 = require("../getSaveType");
const runScript_1 = require("../runScript");
const postInstall_1 = require("../install/postInstall");
const extendOptions_1 = require("./extendOptions");
const pnpmPkgJson_1 = require("../pnpmPkgJson");
const lock_1 = require("./lock");
const graphController_1 = require("../fs/graphController");
const storeController_1 = require("../fs/storeController");
const shrinkwrap_1 = require("../fs/shrinkwrap");
const modulesController_1 = require("../fs/modulesController");
const flattenDependencies_1 = require("../install/flattenDependencies");
function install(maybeOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        const opts = extendOptions_1.default(maybeOpts);
        const ctx = yield getContext_1.default(opts);
        const installCtx = yield createInstallCmd(opts, ctx.graph, ctx.shrinkwrap, ctx.cache);
        if (!ctx.pkg)
            throw runtimeError_1.default('No package.json found');
        const packagesToInstall = Object.assign({}, ctx.pkg.dependencies || {});
        if (!opts.production)
            Object.assign(packagesToInstall, ctx.pkg.devDependencies || {});
        return lock_1.default(ctx.storePath, () => installInContext('general', packagesToInstall, ctx, installCtx, opts));
    });
}
exports.install = install;
/**
 * Perform installation.
 *
 * @example
 *     install({'lodash': '1.0.0', 'foo': '^2.1.0' }, { silent: true })
 */
function installPkgs(fuzzyDeps, maybeOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        let packagesToInstall = mapify(fuzzyDeps);
        if (!Object.keys(packagesToInstall).length) {
            throw new Error('At least one package has to be installed');
        }
        const opts = extendOptions_1.default(maybeOpts);
        const ctx = yield getContext_1.default(opts);
        const installCtx = yield createInstallCmd(opts, ctx.graph, ctx.shrinkwrap, ctx.cache);
        return lock_1.default(ctx.storePath, () => installInContext('named', packagesToInstall, ctx, installCtx, opts));
    });
}
exports.installPkgs = installPkgs;
function installInContext(installType, packagesToInstall, ctx, installCtx, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: ctx.graph should not be muted. installMultiple should return a new graph
        const oldGraph = cloneDeep(ctx.graph);
        const nodeModulesPath = path.join(ctx.root, 'node_modules');
        const client = new RegClient(adaptConfig(opts));
        const pkgs = yield lock_1.default(ctx.cache, function () {
            return __awaiter(this, void 0, void 0, function* () {
                const installOpts = {
                    linkLocal: opts.linkLocal,
                    dependent: ctx.root,
                    root: ctx.root,
                    storePath: ctx.storePath,
                    force: opts.force,
                    depth: opts.depth,
                    tag: opts.tag,
                    engineStrict: opts.engineStrict,
                    nodeVersion: opts.nodeVersion,
                    preserveSymlinks: opts.preserveSymlinks,
                    got: got_1.default(client, {
                        cachePath: ctx.cache,
                        cacheTTL: opts.cacheTTL
                    }),
                    fetchingFiles: Promise.resolve(),
                    nodeModulesStore: path.join(nodeModulesPath, '.resolutions'),
                };
                return yield installMultiple_1.default(installCtx, packagesToInstall, ctx.pkg && ctx.pkg && ctx.pkg.optionalDependencies || {}, nodeModulesPath, installOpts);
            });
        });
        if (opts.flatTree) {
            pnpm_logger_1.default.info('Flattening the dependency tree');
            yield flattenDependencies_1.default(ctx.root, ctx.storePath, pkgs, ctx.graph);
        }
        if (installType === 'named') {
            const saveType = getSaveType_1.default(opts);
            if (saveType) {
                if (!ctx.pkg) {
                    throw new Error('Cannot save because no package.json found');
                }
                const inputNames = Object.keys(packagesToInstall);
                const savedPackages = pkgs.filter((pkg) => inputNames.indexOf(pkg.pkg.name) > -1);
                const pkgJsonPath = path.join(ctx.root, 'package.json');
                yield save_1.default(pkgJsonPath, savedPackages, saveType, opts.saveExact);
            }
        }
        const newGraph = Object.assign({}, ctx.graph);
        yield removeOrphanPkgs(oldGraph, newGraph, ctx.root, ctx.storePath);
        yield graphController_1.save(path.join(ctx.root, 'node_modules'), newGraph);
        yield shrinkwrap_1.save(ctx.root, ctx.shrinkwrap);
        if (ctx.isFirstInstallation) {
            yield modulesController_1.save(path.join(ctx.root, 'node_modules'), {
                packageManager: `${pnpmPkgJson_1.default.name}@${pnpmPkgJson_1.default.version}`,
                storePath: ctx.storePath,
                type: opts.flatTree ? 'flat' : 'nested',
            });
        }
        yield linkPeers_1.default(installCtx.installs);
        // postinstall hooks
        if (!(opts.ignoreScripts || !installCtx.installationSequence || !installCtx.installationSequence.length)) {
            yield seq(installCtx.installationSequence.map((pkgId) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield postInstall_1.default(installCtx.installs[pkgId].hardlinkedLocation, installLogger(pkgId));
                }
                catch (err) {
                    if (installCtx.installs[pkgId].optional) {
                        pnpm_logger_1.default.warn({
                            message: `Skipping failed optional dependency ${pkgId}`,
                            err,
                        });
                        return;
                    }
                    throw err;
                }
            })));
        }
        if (!opts.ignoreScripts && ctx.pkg) {
            const scripts = ctx.pkg && ctx.pkg.scripts || {};
            if (scripts['postinstall']) {
                npmRun('postinstall', ctx.root);
            }
            if (installType === 'general' && scripts['prepublish']) {
                npmRun('prepublish', ctx.root);
            }
        }
    });
}
function removeOrphanPkgs(oldGraphJson, newGraphJson, root, storePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const oldPkgIds = new Set(Object.keys(oldGraphJson));
        const newPkgIds = new Set(Object.keys(newGraphJson));
        const store = (yield storeController_1.read(storePath)) || {};
        const notDependents = difference(oldPkgIds, newPkgIds);
        yield Promise.all(Array.from(notDependents).map(function (notDependent) {
            return __awaiter(this, void 0, void 0, function* () {
                if (store[notDependent]) {
                    store[notDependent].splice(store[notDependent].indexOf(root), 1);
                    if (!store[notDependent].length) {
                        delete store[notDependent];
                        yield rimraf(path.join(storePath, notDependent));
                    }
                }
            });
        }));
        const newDependents = difference(newPkgIds, oldPkgIds);
        newDependents.forEach(newDependent => {
            store[newDependent] = store[newDependent] || [];
            if (store[newDependent].indexOf(root) === -1) {
                store[newDependent].push(root);
            }
        });
        yield storeController_1.save(storePath, store);
    });
}
function difference(setA, setB) {
    const difference = new Set(setA);
    for (const elem of setB) {
        difference.delete(elem);
    }
    return difference;
}
function createInstallCmd(opts, graph, shrinkwrap, cache) {
    return __awaiter(this, void 0, void 0, function* () {
        return {
            fetchLocks: {},
            installLocks: {},
            installs: {},
            graph,
            shrinkwrap,
            resolutionLinked: {},
            installed: new Set(),
            installationSequence: [],
        };
    });
}
function adaptConfig(opts) {
    const registryLog = pnpm_logger_1.default('registry');
    return {
        proxy: {
            http: opts.proxy,
            https: opts.httpsProxy,
            localAddress: opts.localAddress
        },
        ssl: {
            certificate: opts.cert,
            key: opts.key,
            ca: opts.ca,
            strict: opts.strictSsl
        },
        retry: {
            count: opts.fetchRetries,
            factor: opts.fetchRetryFactor,
            minTimeout: opts.fetchRetryMintimeout,
            maxTimeout: opts.fetchRetryMaxtimeout
        },
        userAgent: opts.userAgent,
        log: Object.assign({}, registryLog, {
            verbose: registryLog.debug.bind(null, 'http'),
            http: registryLog.debug.bind(null, 'http'),
        }),
        defaultTag: opts.tag
    };
}
function npmRun(scriptName, pkgRoot) {
    const result = runScript_1.sync('npm', ['run', scriptName], {
        cwd: pkgRoot,
        stdio: 'inherit'
    });
    if (result.status !== 0) {
        process.exit(result.status);
    }
}
const lifecycleLogger = pnpm_logger_1.default('lifecycle');
function installLogger(pkgId) {
    return (stream, line) => {
        const logLevel = stream === 'stderr' ? 'error' : 'info';
        lifecycleLogger[logLevel]({ pkgId, line });
    };
}
function mapify(pkgs) {
    if (!pkgs)
        return {};
    if (Array.isArray(pkgs)) {
        return pkgs.reduce((pkgsMap, pkgRequest) => {
            const matches = /(@?[^@]+)@(.*)/.exec(pkgRequest);
            if (!matches) {
                pkgsMap[pkgRequest] = '*';
            }
            else {
                pkgsMap[matches[1]] = matches[2];
            }
            return pkgsMap;
        }, {});
    }
    return pkgs;
}
//# sourceMappingURL=install.js.map